/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2000, 2010 Oracle and/or its affiliates.
 *
 * OpenOffice.org - a multi-platform office productivity suite
 *
 * This file is part of OpenOffice.org.
 *
 * OpenOffice.org is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * OpenOffice.org is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with OpenOffice.org.  If not, see
 * <http://www.openoffice.org/license.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/

#ifndef SC_MARKDATA_HXX
#define SC_MARKDATA_HXX

#include "address.hxx"
#include <tools/solar.h>
#include "scdllapi.h"

class ScMarkArray;
class ScRangeList;

//!		todo:
//!		It should be possible to have MarkArrays for each table, in order to 
//!		enable "search all" across more than one table again!


class SC_DLLPUBLIC ScMarkData
{
private:
    ScRange			aMarkRange;				// area
    ScRange			aMultiRange;			// maximum area altogether
    ScMarkArray*	pMultiSel;				// multi selection
    sal_Bool			bTabMarked[MAXTABCOUNT];// table marked
    sal_Bool			bMarked;				// rectangle marked
    sal_Bool			bMultiMarked;			

    sal_Bool			bMarking;				// area is being marked -> no MarkToMulti
    sal_Bool			bMarkIsNeg;				// cancel if multi selection

public:
                ScMarkData();
                ScMarkData(const ScMarkData& rData);
                ~ScMarkData();

    ScMarkData&	operator=(const ScMarkData& rData);

    void		ResetMark();
    void		SetMarkArea( const ScRange& rRange );

    void		SetMultiMarkArea( const ScRange& rRange, sal_Bool bMark = sal_True );

    void		MarkToMulti();
    void		MarkToSimple();

    sal_Bool		IsMarked() const				{ return bMarked; }
    sal_Bool		IsMultiMarked() const			{ return bMultiMarked; }

    void		GetMarkArea( ScRange& rRange ) const;
    void		GetMultiMarkArea( ScRange& rRange ) const;

    void		SetAreaTab( SCTAB nTab );

    void		SelectTable( SCTAB nTab, sal_Bool bNew )		{ bTabMarked[nTab] = bNew; }
    sal_Bool		GetTableSelect( SCTAB nTab ) const			{ return bTabMarked[nTab]; }

    void		SelectOneTable( SCTAB nTab );
    SCTAB		GetSelectCount() const;
    SCTAB		GetFirstSelected() const;

    void		SetMarkNegative( sal_Bool bFlag )	{ bMarkIsNeg = bFlag; }
    sal_Bool		IsMarkNegative() const			{ return bMarkIsNeg;  }
    void		SetMarking( sal_Bool bFlag )		{ bMarking = bFlag;   }
    sal_Bool		GetMarkingFlag() const			{ return bMarking;	  }

    //	for FillInfo / Document etc.
    const ScMarkArray* GetArray() const			{ return pMultiSel; }

    sal_Bool		IsCellMarked( SCCOL nCol, SCROW nRow, sal_Bool bNoSimple = false ) const;
    void		FillRangeListWithMarks( ScRangeList* pList, sal_Bool bClear ) const;
    void		ExtendRangeListTables( ScRangeList* pList ) const;

    void		MarkFromRangeList( const ScRangeList& rList, sal_Bool bReset );

    SCCOLROW	GetMarkColumnRanges( SCCOLROW* pRanges );
    SCCOLROW	GetMarkRowRanges( SCCOLROW* pRanges );

    sal_Bool		IsColumnMarked( SCCOL nCol ) const;
    sal_Bool		IsRowMarked( SCROW nRow ) const;
    sal_Bool		IsAllMarked( const ScRange& rRange ) const;		// Multi

                /// May return -1
    SCsROW		GetNextMarked( SCCOL nCol, SCsROW nRow, sal_Bool bUp ) const;
    sal_Bool		HasMultiMarks( SCCOL nCol ) const;
    sal_Bool		HasAnyMultiMarks() const;

    //	adjust table marking:
    void		InsertTab( SCTAB nTab );
    void		DeleteTab( SCTAB nTab );
};



#endif


/* vim:set shiftwidth=4 softtabstop=4 expandtab: */
