/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2000, 2010 Oracle and/or its affiliates.
 *
 * OpenOffice.org - a multi-platform office productivity suite
 *
 * This file is part of OpenOffice.org.
 *
 * OpenOffice.org is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * OpenOffice.org is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with OpenOffice.org.  If not, see
 * <http://www.openoffice.org/license.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/

#ifndef SC_SIMPREF_HXX
#define SC_SIMPREF_HXX

#include <vcl/group.hxx>

#include <vcl/combobox.hxx>

#include <vcl/fixed.hxx>

#include <vcl/morebtn.hxx>
#include "anyrefdg.hxx"
#include "dbcolect.hxx"
#include "expftext.hxx"

class ScViewData;
class ScDocument;


//============================================================================

class ScSimpleRefDlg: public ScAnyRefDlg
{
private:
    Link			aCloseHdl;
    Link			aDoneHdl;
    Link			aAbortedHdl;
    Link			aChangeHdl;

    FixedText		aFtAssign;
    formula::RefEdit		aEdAssign;
    formula::RefButton		aRbAssign;

    OKButton		aBtnOk;
    CancelButton	aBtnCancel;
    HelpButton		aBtnHelp;

    ScViewData*		pViewData;
    ScDocument*		pDoc;
    sal_Bool			bRefInputMode;

    ScRange			theCurArea;
    sal_Bool			bCloseFlag;
    sal_Bool			bAutoReOpen;
    sal_Bool			bCloseOnButtonUp;
    sal_Bool			bSingleCell;
    sal_Bool            bMultiSelection;

    void			Init();

    DECL_LINK( CancelBtnHdl, void * );
    DECL_LINK( OkBtnHdl, void * );


protected:

    virtual void	RefInputDone( sal_Bool bForced = false );

public:
                    ScSimpleRefDlg( SfxBindings* pB, SfxChildWindow* pCW, Window* pParent,
                                 ScViewData*	ptrViewData );
                    ~ScSimpleRefDlg();

    virtual void	SetReference( const ScRange& rRef, ScDocument* pDoc );

    virtual sal_Bool	IsRefInputMode() const;
    virtual void	SetActive();
    virtual sal_Bool	Close();

    void			StartRefInput();

    virtual	String	GetRefString() const;
    virtual	void	SetRefString(const String &rStr);
    virtual void    FillInfo(SfxChildWinInfo&) const;

    void			SetCloseHdl( const Link& rLink );
    void			SetUnoLinks( const Link& rDone, const Link& rAbort,
                                const Link& rChange );

    void            SetFlags( sal_Bool bSetCloseOnButtonUp, sal_Bool bSetSingleCell, sal_Bool bSetMultiSelection );

    void			SetAutoReOpen(sal_Bool bFlag) {bAutoReOpen=bFlag;}
};



#endif // SC_DBNAMDLG_HXX

/* vim:set shiftwidth=4 softtabstop=4 expandtab: */
